from textual.widgets import MarkdownViewer, Button, Footer, Header, Static
from textual.app import  ComposeResult
from textual.screen import Screen
from textual.widgets import Footer
from textual.containers import Container, VerticalScroll, Horizontal, Grid
from textual import  on, work
import pandas as pd
from .revertpaxmodule import apiPaxFunctions, PushConfigs, updateTerminals 
from .ti_labels_iccid import create_pdf
from .confmscn import Confirm_Screen
from .singleTerminal import parseList




def update_table(appdata):
        headers = "| ".join(appdata[0].keys()) + " \n| --------------- | --------------- | --------------- | --------------- | --------------- |"
        rows = "\n|".join(["|".join(str(value) for value in item.values()) + "|" for item in appdata])
        content = f"## Installed Applications\n\n|{headers} \n|{rows}"
        return content

def format_terminal_details(termDetails:dict, appdata:list):

    text = ""
    text += "## Basic Info\n\n"
    for key, value in termDetails.items():
        if key == "serialNo":
            text += f"**Serial No:** {value}\n"
    for key, value in termDetails.items(): 
            if key !="serialNo":
                text += f"* {key}: {value}\n"
    text += update_table(appdata) 
    return text

class TerminalDetailsScreen(Screen):
    
    BINDINGS = [("escape", "app.pop_screen", "BACK")]
    CSS_PATH = "singleTermDetails.tcss"
    
    def __init__(self,details:tuple):

        self.termdetails = details[0]
        self.df = pd.DataFrame(details[0])
        self.appdata = details[1]
        self.markdown = format_terminal_details(*self.termdetails,self.appdata)
        self.button_list = [
            {'name':'Reset Terminal', 'id': 'reset', 'classes':'gtask-buttons'},
            {'name':'Activate Terminal','id':'activate','classes':'gtask-buttons'},
            {'name':'Deactivate','id':'deactivate','classes':'gtask-buttons'},
            {'name':'Delete', 'id':'delete', 'classes':'gtask-buttons'},
            {'name':'Refresh Terminal Detials','id':'details','classes':'gtask-buttons'},
            {'name':'Create Label', 'id':'labels', 'classes':'gtask-buttons'}
        ]
        self.op = apiPaxFunctions()
        self.configop = PushConfigs()
        self.operations = {
            "activate": self.op.activateTerminals,
            "details": self.op.startPaxGroup,
            "deactivate": self.op.disableTerminals, 
            "move": None,
            "delete": self.op.deleteTerminals,
            "reset": updateTerminals,
            "labels": create_pdf,
            "payanywhere": self.configop.pushPAConfig,
            "broadpos":self.configop.pushBroadPosEPX,
            "other": self.configop.pushBroadPos_nonEPX
        }
        self.functionDict = {}
        super().__init__()

    def compose(self)-> ComposeResult:
        yield Header(name='PaxTools')
        with Container(id="app-grid"):
            with VerticalScroll(id = "top-pane"):
                yield MarkdownViewer(markdown=self.markdown,show_table_of_contents= True, id="mkdn") 
            with VerticalScroll(id = "bottom-left"):
                yield Static("Available Tasks", classes="titleheader")
                for button in self.button_list:
                    yield Button(button['name'], id=button['id'], classes=button['classes']) # type: ignore
            with VerticalScroll(id= "bottom-right"):
                yield Static("Configuration Tasks", classes="titleheader")
                yield Button("Config for PayAnywhere", id="payanywhere", classes="buttons")
                yield Button("Config for BroadPOS - EPX", id="broadpos",classes="buttons")
                yield Button("Config for BroadPOS - Not EPX", id ="other", classes="buttons")
        yield Footer()

    @on(Button.Pressed)
    @work
        
    async def do_stuff(self, event: Button.Pressed):
        operation = self.operations.get(event.button.id)  # type: ignore
        if await self.app.push_screen_wait(Confirm_Screen("Please confirm terminal network connection and open PaxStore client on device.")):
            result = await operation(idList = self.df['id'], serialNoList = self.df['serialNo'], df = self.df)  # type: ignore
            self.notify(str(result))
            if event.button.id == "details":
                resp = await parseList(self.df['serialNo'])
                updateTermDetails = resp[0]
                updateAPKinfo = resp[1]
                self.markdown=format_terminal_details(*updateTermDetails,updateAPKinfo)  


from .revertpaxmodule import apiPaxFunctions, terminalDetails, getInstalledConfig
import pandas as pd



async def parseList(serialNoList)->dict:
    thing = apiPaxFunctions()
    termDetail = await thing.startPaxGroup(serialNoList)
    config = await getInstalledConfig(serialNoList)
    for item in config:
        apklist = item['installedApks']
        df = pd.DataFrame(apklist)
    details = await terminalDetails(detail['id']for detail in config)
    termDetails_dict = termDetail.to_dict('records')

    return termDetails_dict, apklist

async def parseApk(serialNoList):
    config = await getInstalledConfig(serialNoList)
    for item in config:
        apklist = item['installedApks']
        df = pd.DataFrame(apklist)
    return df


class TerminalNotFoundError(Exception):
    """Exception raised when a terminal is not found in PaxStore."""
    def __init__(self, serial_no):
        super().__init__(f"Terminal SN {serial_no} not found in PaxStore.")
        self.serial_no = serial_no


from textual.validation import Function, Number, ValidationResult, Validator

EXAMPLES = [{'Model': 'A920', 'ExampleSerialNo': '0821674609'}, {'Model': 'A920Pro', 'ExampleSerialNo': '1851160310'}, {'Model': 'A35', 'ExampleSerialNo': '2290065118'}, {'Model': 'A60', 'ExampleSerialNo': '1350021966'}, {'Model': 'A77', 'ExampleSerialNo': '1760234077'}, {'Model': 'A80', 'ExampleSerialNo': '1240383205'}, {'Model': 'E500', 'ExampleSerialNo': '1150050557'}, {'Model': 'E600', 'ExampleSerialNo': '1190018657'}, {'Model': 'E700', 'ExampleSerialNo': '1340004658'}, {'Model': 'E600M', 'ExampleSerialNo': '2270008806'}, {'Model': 'Q10A', 'ExampleSerialNo': '2400004913'}, {'Model': 'Q20L', 'ExampleSerialNo': '1140077754'}, {'Model': 'D135', 'ExampleSerialNo': '1890200678'}, {'Model': 'S300', 'ExampleSerialNo': '5G047654'}, {'Model': 'SP30', 'ExampleSerialNo': '3L013509'}]

class SerialNoValidator(Validator):
    def validate(self, value: str) -> ValidationResult:
        
        if value in ["0000", "BKSPC"]:
            return self.success()
        if self.is_serialNo(value):
            return self.success()
        else:
            return self.failure("Invalid Entry")

    @staticmethod
    def is_serialNo(value: str) -> bool:
        """Checks if the given value is a valid serial number.

        Args:
            value (str): The value to check.
            examples (list[dict]): A list of dictionaries containing 'ExampleSerialNo' keys.

        Returns:
            bool: True if the value is a valid serial number, False otherwise.
        """
        
        # Ensure length is between 8 and 10 characters
        if not (8 <= len(value) <= 10):
            return False
        
        # Check first three digits against examples
        for example in EXAMPLES:
            if value[:3] == example['ExampleSerialNo'][:3]:
                return True
            
        # If no match found, return False
        return False


"""def validTest():
    
    check = SerialNoValidator()
    thing = check.validate('0821674609')
    print(thing.is_valid)

validTest()"""


import pandas as pd
from textual.app import ComposeResult
from textual.screen import Screen
from textual.widgets import Static, Button, Header, Footer 
from textual import  on, work
from .DFTable import DataFrameTable
from .revertpaxmodule import apiPaxFunctions, PushConfigs, terminalDetails
from .ti_labels_iccid import create_pdf
from textual.containers import Container, VerticalScroll
from .confmscn import Confirm_Screen
from .singleTerminal import parseList
from .SingleTermDetailsScreen import TerminalDetailsScreen

class FunctionsScreen(Screen):

    BINDINGS = [("escape", "app.pop_screen", "BACK")]
    CSS_PATH = "css_lib/group_gunctions.tcss"

    def __init__(self, df:pd.DataFrame) -> None:
        self.df = df
        self.button_list = [
            {'name':'Activate Group','id':'activate','classes':'gtask-buttons'},
            {'name':'Get Terminal Detials','id':'details ','classes':'gtask-buttons'},
            {'name':'Deactivate','id':'deactivate','classes':'gtask-buttons'},
            {'name':'Move Terminals', 'id':'move', 'classes':'gtask-buttons'},
            {'name':'Delete Group', 'id':'delete', 'classes':'gtask-buttons'},
            {'name':'Create Ticket Labels', 'id':'labels', 'classes':'gtask-buttons'}
        ]
        self.op = apiPaxFunctions()
        self.configOps = PushConfigs()
        self.operations = {
            "activate": self.op.activateTerminals,
            "details": terminalDetails,
            "deactivate": self.op.disableTerminals, 
            "move": None,
            "delete": self.op.deleteTerminals,
            "labels": create_pdf,
            "payanywhere": self.configOps.pushPAConfig,
            "broadpos":self.configOps.pushBroadPosEPX,
            "other": self.configOps.pushBroadPos_nonEPX
        }

        
        self.functionDict = {}
        super().__init__()
    
    def compose(self) -> ComposeResult:
        yield Header()
        with Container(id="app-grid"):
            with VerticalScroll(id = "top-pane"):
                yield DataFrameTable()
            with VerticalScroll(id = "bottom-left"):
                for button in self.button_list:
                    yield Button(button['name'], id=button['id'], classes=button['classes']) # type: ignore
            with VerticalScroll(id= "bottom-right"):
                yield Static("Configuration Tasks", classes="titleheader")
                yield Button("Config for PayAnywhere", id="payanywhere", classes="buttons")
                yield Button("Config for BroadPOS - EPX", id="broadpos",classes="buttons")
                yield Button("Config for BroadPOS - Not EPX", id ="other", classes="buttons")
        yield Footer()
    async def on_mount(self):   
        """if self.df == None:
            self.mdf = pd.read_pickle("/Users/scottjamnik/python-to-google-sheets/E600M04.29.2024.14.051.pkl")
        else: self.mdf = pd.DataFrame(self.df)"""

        self.table = self.query_one(DataFrameTable)
        self.table.add_df(self.df)
        self.app.notify(str(self.df))

    
    async def do_stuff(self, event: Button.Pressed):
        operation = self.operations.get(event.button.id)  # type: ignore
        if await self.app.push_screen_wait(Confirm_Screen("Please confirm terminal network connection and open PaxStore client on device.")):
            result = await operation(idList = self.df['id'], serialNoList = self.df['serialNo'], df = self.df)  # type: ignore
            self.notify(str(result))
            if event.button.id == "details":
                self.ndf = pd.DataFrame(result).drop_duplicates(subset=["serialNo"])
                self.table.update_df(self.ndf)
                self.df = self.ndf 
            
    @work
    @on(DataFrameTable.CellSelected)
    async def note_cell(self, event:DataFrameTable.CellSelected):
        if event.value in self.df['serialNo'].values:
            self.app.notify(str(event.value))
            if await self.app.push_screen_wait(Confirm_Screen(message=f"View {event.value} Terminal Page?")):
                dList = [event.value]
                termDetails = await parseList(dList)
                self.app.notify(str(termDetails))            
                self.app.push_screen(TerminalDetailsScreen(termDetails))   


from textual.screen import ModalScreen
from textual.app import  ComposeResult

from textual.widgets import Static, Input, Header,Footer
from textual import on, work
import pandas as pd
from .revertpaxmodule import apiPaxFunctions, findGroup  # Assuming this is in the same directory
from .confmscn import Confirm_Screen  # Assuming this is in the same directory
from .functionsScreen import FunctionsScreen
from .serialNoValidator import SerialNoValidator
from .tui_exceptions import TerminalNotFoundError

class Scan_serials(ModalScreen):

    """SERIAL NUMBER INPUT"""
    BINDINGS = [("escape", "app.pop_screen", "Go Back"),("0000", "", "Complete Entry"),("BKSPC", "", "Del item")]


    def __init__(self):
        self.serialNoList = []
        self.copySerialNoList = self.serialNoList
        self.validator = SerialNoValidator()  # Create an instance of the validator
        super().__init__()
    
    def compose(self) -> ComposeResult:
        
        yield Header(name='PaxTools')
        yield Static("SCAN OR TYPE SERIAL NUMBER. KEY IN 0000 TO COMPLETE. KEY IN BKSPC TO REMOVE PREVIOUS ENTRY:")
        yield Input(placeholder="S/N",validators=[self.validator])
        yield Footer()
    
    @on(Input.Submitted)
    @work
    async def update_serial_list(self):
        user_input = self.query_one(Input)
        serialNo = user_input.value
        self.serialNoList.append(serialNo)
        self.mount(Static(serialNo))
        if user_input.value == "BKSPC":
            self.serialNoList.pop()
            self.serialNoList.pop()
        if ":" in user_input.value:
            self.serialNoList.pop()
            self.app.bell()
        if user_input.value == "0000":
            self.disabled = True
            self.app.bell()
            self.serialNoList.pop()
            sndf = pd.DataFrame({"serialNo":self.serialNoList})
            if await self.app.push_screen_wait(Confirm_Screen(f"Are these terminals you wish to activate\n{self.serialNoList}?")):
                self.app.notify("SEARCHING...")
                try:
                    self.group = await findGroup(self.serialNoList)
                    self.app.push_screen(FunctionsScreen(pd.DataFrame(self.group)))
                except TerminalNotFoundError as e:
                    # Display error message and prompt for registration
                    not_found_terminals = [e.serial_no]
                    self.copySerialNoList.remove(e.serial_no)
                    try:
                        self.group = await findGroup(self.copySerialNoList)
                    except TerminalNotFoundError as e:
                        not_found_terminals.append(e.serial_no)
                    self.app.notify(str(len(not_found_terminals)))
                    if len(not_found_terminals) == 1:
                        error_message = f"Terminal not found: {not_found_terminals[0]}\nRegister now?"
                    else:
                        error_message = f"Terminals not found: {', '.join(not_found_terminals)}\nRegister now?"   
                    if await self.app.push_screen_wait(Confirm_Screen(error_message)):
                        func = apiPaxFunctions()
                        add = await func.createTerminals(serialNoList=not_found_terminals)
                        self.app.notify(f"{not_found_terminals} added successfully") 
                        self.group = await findGroup(sndf['serialNo'])
                        df = pd.DataFrame(self.group)
                        self.app.push_screen(FunctionsScreen(df))
        user_input.clear()